package org.one.stone.soup.file;

import java.io.File;
import java.io.IOException;
import java.util.Enumeration;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

public class UnZipper {
	
	private ZipFile zipFile;
	private String[] excludeList=null;
	
	public UnZipper(String fileName) throws IOException
	{
		zipFile = new ZipFile( new File(fileName) );
	}
	
	public void unzipAll(String targetFolder) throws IOException
	{
		Enumeration entries = zipFile.entries();
		
		while(entries.hasMoreElements())
		{
			ZipEntry entry = (ZipEntry)entries.nextElement();
			if(isExcluded(entry.getName()))
			{
				continue;
			}
			
			if(	entry.isDirectory() )
			{
				new File(
						targetFolder+"/"+entry.getName()
					).mkdirs();
			}
			else
			{
				FileHelper.copy(
						zipFile.getInputStream(entry),
						targetFolder+"/"+entry.getName()
					);
			}
		}
	}
	
	public void unzipAllAndBackUp(String targetFolder,String backupFile) throws IOException
	{
		Zipper backup = new Zipper(backupFile);
		Enumeration entries = zipFile.entries();
		
		while(entries.hasMoreElements())
		{
			ZipEntry entry = (ZipEntry)entries.nextElement();
			if(isExcluded(entry.getName()))
			{
				continue;
			}
			
			if(	entry.isDirectory() )
			{
				new File(
						targetFolder+"/"+entry.getName()
					).mkdirs();
			}
			else
			{
				File file = new File(targetFolder+"/"+entry.getName());
				if( file.exists() )
				{
					backup.addEntry(entry.getName(),file.getAbsolutePath());
				}
				
				FileHelper.copy(
						zipFile.getInputStream(entry),
						targetFolder+"/"+entry.getName()
					);
			}
		}
		
		backup.close();
	}
	
	public void setExcludeList(String[] excludeList)
	{
		this.excludeList = excludeList;
	}
	
	private boolean isExcluded(String entryName)
	{
		if(excludeList==null)
		{
			return false;
		}
		
		for(int loop=0;loop<excludeList.length;loop++)
		{
			if(entryName.indexOf(excludeList[loop])==0)
			{
				return true;
			}
		}
		return false;
	}
}
