package org.one.stone.soup.net;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.Socket;

import org.one.stone.soup.io.Connection;
import org.one.stone.soup.io.ConnectionListener;
import org.one.stone.soup.io.StreamConnection;
import org.one.stone.soup.util.TimeWatch;

public class SocketConnection implements Connection,ActionListener{

	private long TIMEOUT = 60000;

	private TimeWatch timer; 

	private Socket socket1;
	private Socket socket2;

	private ConnectionListener listener;

	private String alias;

	private StreamConnection up;
	private StreamConnection down;

	public SocketConnection( Socket socket2,String alias ) throws IOException
	{
		this.alias = alias;
		this.socket2 = socket2;		
	}
	public void connect( Socket socket2,ConnectionListener listener ) throws IOException
	{
		this.listener = listener;
		this.socket2 = socket2;
		
		initialise();		
	}
	
	public SocketConnection( Socket socket1,Socket socket2,String alias,ConnectionListener listener ) throws IOException
	{
		this.alias = alias;
		this.listener = listener;
		this.socket1 = socket1;
		this.socket2 = socket2;
		
		initialise();
	}
	
	private void initialise() throws IOException
	{	
		up = new StreamConnection(socket1.getInputStream(),socket2.getOutputStream(),alias+".up",10000);
		down = new StreamConnection(socket2.getInputStream(),socket1.getOutputStream(),alias+".down",10000);

		listener.upStreamConnectionCreated(up);
		listener.downStreamConnectionCreated(down);
		
		up.connect(listener,"Up Stream Connection for "+alias );
		down.connect(listener,"Down Stream Connection for "+alias );
		
		timer = new TimeWatch(1000);
		timer.addActionListener(this);
		timer.setActionCommand("checkAlive");
		timer.startTimer();		
	}

	public String getAlias()
	{
		return alias;
	}

	public long getDataSentSize()
	{
		return up.getDataSentSize();
	}

	public long getDataReceivedSize()
	{
		return down.getDataSentSize();
	}

	public void close()
	{
		try{
			socket1.close();
		}
		catch(Exception e){}
		
		try{
			socket2.close();
		}
		catch(Exception e){}		
		
		timer.stopTimer();
		up.disconnect();
		down.disconnect();
	}

	public void actionPerformed(ActionEvent event)
	{
	    checkAlive();
	}
	
	public void checkAlive()
	{
		if(System.currentTimeMillis()-up.getLastWriteTime()>TIMEOUT && System.currentTimeMillis()-down.getLastWriteTime()>TIMEOUT)
		{
		    System.out.println("Closed Socket Connection from "+socket1.getInetAddress()+" to "+socket2.getInetAddress());
		    try{
		    	listener.connectionClosed( this );
		    }
		    catch(Exception e){}
		}
	}
	
	public InputStream getInputStream() throws IOException
	{
		return socket2.getInputStream();
	}
	
	public OutputStream getOutputStream() throws IOException
	{
		return socket2.getOutputStream();
	}
}
