package org.one.stone.soup.reloader;

import java.io.File;
import java.lang.reflect.Method;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.Vector;

import org.one.stone.soup.file.FileChangeListener;
import org.one.stone.soup.file.FilesWatch;
import org.one.stone.soup.stringhelper.FileNameHelper;

public class Reloader implements FileChangeListener{

	private ClassLoader classLoader;
	private String jarPath;
	private FilesWatch jarWatcher;
	
	private String[] args;
	private String rootClass;

	private Object reloadListener;
	
	public static void main(String[] args)
	{
		String path = args[0];
		String clazz = args[1];
		String[] newArgs = new String[args.length-2];
		System.arraycopy(args,2,newArgs,0,newArgs.length);
		
		try{
			new Reloader(path,clazz,newArgs);
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}
	
	public Reloader(String jarPath,String rootClass,String[] args) throws Exception
	{
		this.rootClass = rootClass;
		this.jarPath = jarPath;
		this.args = args;
		
		jarWatcher = new FilesWatch(jarPath,5000);
		jarWatcher.addFolder( new File(jarPath) );
		jarWatcher.addListener(this);
		
		reload();
			
	}
	
	public void reload() throws Exception
	{
		File[] files = new File(jarPath).listFiles();
		Vector jars = new Vector();
		for(int loop=0;loop<files.length;loop++)
		{
			if( FileNameHelper.getExt( files[loop].getName() ).equals("jar") )
			{
				URL jar = files[loop].toURI().toURL();
				jars.addElement( jar );
				System.out.println("Added jar "+jar);
			}
		}
		URL[] urls = (URL[])jars.toArray(new URL[0]);
		
		classLoader = new URLClassLoader( urls );		
		
		Class clazz = Class.forName(rootClass,true,classLoader);
		Method method = clazz.getMethod("main",new Class[]{String[].class});
		method.invoke(null,new Object[]{args});
		
		System.out.println("Invoked "+rootClass);
		
		try
		{
			reloadListener = clazz.getMethod("getReloadListener",new Class[]{}).invoke(null,new Object[]{});
			System.out.println(" and added as reload listener "+reloadListener+".");
		}
		catch(Exception e)
		{
			System.out.println(" no reload listener available.");
		}
	}

	public void fileChanged(File file) {
		
		System.out.println("Code has changed. Reloading");
		boolean reload = true;
		if(reloadListener!=null)
		{
			try{
				reloadListener.getClass().getMethod("codeHasChangedReload",new Class[]{}).invoke(reloadListener,new Object[]{});
				System.out.println("Reload Listener informed.");
			}
			catch(Exception e)
			{
				System.out.println("Reload Listener NOT informed. Call to codeHasChangedReload failed.");
				e.printStackTrace();
			}
		}
		
		if(reload)
		{
			try{
				classLoader=null;
				reloadListener=null;
				System.gc();
				System.gc();
				
				reload();
			}
			catch(Exception e)
			{
				e.printStackTrace();
			}
		}
	}
}
