package org.one.stone.soup.swing;

import java.awt.*;
import javax.swing.*;
import javax.swing.text.DefaultHighlighter;
import javax.swing.text.Highlighter;

import org.one.stone.soup.stringhelper.StringArrayHelper;

/*
 * Wet-Wired.com Library Version 2.1
 *
 * Copyright 2000-2001 by Wet-Wired.com Ltd.,
 * Portsmouth England
 * This software is OSI Certified Open Source Software
 * This software is covered by an OSI approved open source licence
 * which can be found at http://www.wet-wired.co.uk/pages/licence-os.html
 */

/**
 * @author nikcross
 *
 */
public class JTextConsole extends JTextArea {

	private Color[] highlightColor = new Color[]{
											new Color(100,100,250),
											new Color(100,250,100),
											new Color(250,100,100),
											new Color(220,220,80),
											new Color(220,80,220),
											new Color(80,220,220),
											new Color(220,80,220)
										};

public JTextConsole()
{
	this(20,80);
}
public JTextConsole(int rows, int columns) {
	setRows(rows);
	setColumns(columns);
	setEditable(true);
}
/**
 *
 */
public void clear()
{
	setText("");
}

public String getSelected()
{
	return getSelectedText();
}

public void find(String find)
{
	String[] phrases = StringArrayHelper.parseSpacedFields( find );

	Highlighter highlighter = getHighlighter();
	highlighter.removeAllHighlights();

	for(int loop=0;loop<phrases.length;loop++)
	{
		find = phrases[loop];
		Color color = highlightColor[loop%highlightColor.length];

		String text = getText();

		int index = text.indexOf(find);
		int pos = 0;

		while(index!=-1)
		{
			try{
				highlighter.addHighlight(index,index+find.length(),new DefaultHighlighter.DefaultHighlightPainter(color));
			}
			catch(Exception e)
			{
				e.printStackTrace();
			}

			pos=index+find.length();
			index = text.indexOf(find,pos);
			//CodeHighlight highlight = new CodeHighlight(textArea,index,index+find.length(),Color.green);
		}
	}
}

public void findLine(int line)
{
	String[] lineArray = StringArrayHelper.parseFields(getText(),'\n');
	if(line>lineArray.length)
	{
		return;
	}
	
	int startCursor = 0;
	int loop;
	for(loop=0;loop<lineArray.length;loop++)
	{
		if(loop==line)
		{
			break;
		}
		
		startCursor+=lineArray[loop].length()+1;
	}

	Highlighter highlighter = getHighlighter();
	highlighter.removeAllHighlights();
	
	try{
		Color color = highlightColor[0];
		highlighter.addHighlight(startCursor,startCursor+lineArray[loop].length(),new DefaultHighlighter.DefaultHighlightPainter(color));
	}
	catch(Exception e)
	{
		e.printStackTrace();
	}
}

public void replace(String find,String replace)
{
	String text = getText();

	text = text.replaceAll( find,replace );

	setText(text);
}

public void setForeground(Color c)
{
	super.setForeground(c);

	setCaretColor(c);
}

public void setEnabled(boolean editable)
{
	setEditable(editable);
}

public void appendText(String newText)
{
	append(newText);
}

}

