package org.one.stone.soup.wiki.screen.capture;

import java.awt.Rectangle;
import java.awt.Robot;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URL;

import javax.imageio.ImageIO;
import javax.swing.JApplet;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JProgressBar;

import org.one.stone.soup.file.FileHelper;
import org.one.stone.soup.file.ProgressListener;
import org.one.stone.soup.wiki.client.WikiClient;
import org.one.stone.soup.wiki.client.WikiFileWriter;

public class WikiScreenCaptureApplet extends JApplet implements ActionListener{

	private Robot robot;
	private Rectangle recordArea;
	private WikiClient client;
	private JFrame frame;
	private JButton capture;
	private JProgressBar progressBar;
	
	private String url;
	private String pageName;
	private String userId;
	private String password;
	
public WikiScreenCaptureApplet()
{
	try{
	robot = new Robot();
	}
	catch(Exception e)
	{
		JOptionPane.showConfirmDialog(this,e.getMessage());
		return;
	}
	recordArea = new Rectangle( Toolkit.getDefaultToolkit ().getScreenSize() );	
	JButton capture = new JButton("Capture Screen");
	capture.setActionCommand("capture");
	capture.addActionListener(this);
	getContentPane().add(capture);

	doLayout();
	setVisible(true);
}

public void showButton()
{
	if(frame==null)
	{
		frame = new JFrame();
		
		capture = new JButton("Take Snap Shot");
		capture.setActionCommand("snapShot");
		capture.addActionListener(this);
		
		frame.getContentPane().add(capture);
		frame.pack();
		frame.setResizable(false);
	}
	frame.setVisible(true);
	frame.setAlwaysOnTop(true);
	frame.setDefaultCloseOperation(JFrame.HIDE_ON_CLOSE);
}

public void init()
{
	System.out.println("WikiScreenCapture Applet Version 1.0 build 004");
	
	pageName = getParameter("pageName");	
	url = getParameter("url");
	
	if(url==null)
	{
		url = "http://"+this.getCodeBase().getHost();
	}
	if(pageName==null)
	{
		pageName = getDocumentBase().toExternalForm();
		pageName = pageName.substring( url.length() );
	}	
	System.out.println("Capturing to page "+pageName);
	
	progressBar = new JProgressBar();
	progressBar.setIndeterminate(true);
	progressBar.setString("Saving...");
	progressBar.setStringPainted(true);
	
	client = new WikiClient(url,userId,password);
}

public boolean destroy(Object source) {
	return true;
}

public void actionPerformed(ActionEvent event)
{
	String command = event.getActionCommand();
	
	if( command.equals("capture") )
	{
		showButton();
	}
	else if( command.equals("snapShot") )
	{
		frame.remove( capture );
		frame.add( progressBar );
		frame.doLayout();
		
		new CaptureScreen(this);	
	}
}

public class WikiScreenCaptureWriter implements WikiFileWriter,ProgressListener
{
	private BufferedImage image;
	
	public WikiScreenCaptureWriter(BufferedImage image)
	{
		this.image = image;
	}
	
	public void writeFileTo(OutputStream outStream)
	{
		try{
			progressBar.setIndeterminate(true);
			progressBar.setString("Preparing...");
			
			ByteArrayOutputStream preOutputStream = new ByteArrayOutputStream();
			ImageIO.write(image,"PNG",preOutputStream);
			ByteArrayInputStream preInputStream = new ByteArrayInputStream(preOutputStream.toByteArray());
			
			progressBar.setString("Uploading...");
			progressBar.setIndeterminate(false);
			FileHelper.copy( (InputStream)preInputStream,outStream,this,(long)preOutputStream.size() );
			
			outStream.flush();	
			
			progressBar.setIndeterminate(true);
			progressBar.setString("Saveing...");
		}
		catch(Exception e)
		{
			showMessage(e.getMessage());
		}
	}

	public void finished() {
	}

	public void message(String message) {
		progressBar.setString(message);
	}

	public void progress(long position, long end) {
		progressBar.setMaximum( (int)end );
		progressBar.setValue( (int)position );
	}
}
public void captureScreen()
{
	frame.setVisible(false);
	BufferedImage image = robot.createScreenCapture(recordArea);
	frame.setVisible(true);

	String fileName = JOptionPane.showInputDialog("File Name","screen-shot.png");	
	
	boolean saved = client.uploadFile(pageName,fileName,new WikiScreenCaptureWriter(image));
	if(saved==false)
	{
		if(userId==null)
		{
			userId = JOptionPane.showInputDialog(this,"User Id");
			if(userId!=null)
			{
				password = JOptionPane.showInputDialog(this,"Password");
			}
			client.setUserId(userId);
			client.setPassword(password);
			saved = client.uploadFile(pageName,fileName,new WikiScreenCaptureWriter(image));
		}
	}
	
	capture.setText("Take Snap Shot");
	capture.setActionCommand("snapShot");
	
	frame.remove( progressBar );
	frame.add( capture );
	frame.doLayout();

	
	if(saved==true)
	{
		showMessage("Snap Shot Saved");
	}
	else
	{
		showMessage("Saveing Snap Shot Failed");
	}
	try{
		System.out.println("Opening "+new URL(url+pageName));
		this.getAppletContext().showDocument(new URL(url+pageName));
	}
	catch(Exception e)
	{
		showMessage(e.getMessage());
	}
}
	
public void showMessage(String message)
{
	JOptionPane.showMessageDialog(this,message);	
}

}
