package org.one.stone.soup.wiki.upload.applet;

import java.io.IOException;
import java.io.OutputStream;
import java.net.Socket;
import java.net.URL;

import com.sun.java.browser.net.ProxyInfo;
import com.sun.java.browser.net.ProxyService;

public class WikiURLConnection {

	private URL targetUrl;
	private String proxyHost = null;
	private int proxyPort = -1;
	private Socket connection;
	private String authentication;
	
	public WikiURLConnection(URL url)
	{
		this(url,null);
	}
	
	public WikiURLConnection(URL url,String authentication)
	{
		this.authentication = authentication;
		this.targetUrl = url;
		proxyHost = null;
		proxyPort = -1;
	      try {
		    ProxyInfo info[] = ProxyService.getProxyInfo( url );
		    if(info != null && info.length>0 && info[0]!=null)
		    {
			  proxyPort = info[0].getPort();
			  if(proxyPort!=-1)
			  {
				  proxyHost = info[0].getHost();
				  System.out.println("Proxy set as "+proxyHost+":"+proxyPort);
			  }
		    }
		  }catch (Exception ex) {
			  ex.printStackTrace();
	  	  }	
	}
	
	private Socket getSocketForUrl() throws IOException
	{
		String host = targetUrl.getHost();
		int port = targetUrl.getPort();
		if(port==-1)
		{
			port = 80;
		}
		
		return new Socket( host,port );
	}
	
	public Socket openPutConnection(String pageName,String fileName,long length) throws IOException
	{
		connection = getSocketForUrl();
		OutputStream oStream = connection.getOutputStream();
		
		if(proxyHost!=null)
		{
			proxyConnect(oStream);
		}
		
		oStream.write( new String("PUT "+pageName+"/"+fileName+" HTTP/1.0\r\n").getBytes() );
		oStream.write( new String("Content-Length: "+length+"\r\n").getBytes() );
		addStandardHeader(oStream);
		oStream.write( new String("\r\n").getBytes() );
		oStream.flush();
		
		return connection;
	}
	
	public Socket openGetConnection(String pageName,String fileName) throws IOException
	{
		connection = getSocketForUrl();
		OutputStream oStream = connection.getOutputStream();

		if(proxyHost!=null)
		{
			proxyConnect(oStream);
		}
		oStream.write( new String("GET "+pageName+"/"+fileName+" HTTP/1.0\r\n").getBytes() );
		addStandardHeader(oStream);
		oStream.write( new String("\r\n").getBytes() );
		oStream.flush();
		
		return connection;
	}	
	
	public Socket openPostConnection(String pageName,String fileName) throws IOException
	{
		connection = getSocketForUrl();
		OutputStream oStream = connection.getOutputStream();
		
		if(proxyHost!=null)
		{
			proxyConnect(oStream);
		}
		
		oStream.write( new String("POST "+pageName+"/"+fileName+" HTTP/1.0\r\n").getBytes() );
		addStandardHeader(oStream);
		oStream.write( new String("\r\n").getBytes() );
		oStream.flush();
		
		return connection;
	}	
	
	public Socket openDeleteConnection(String pageName,String fileName) throws IOException
	{
		connection = getSocketForUrl();
		OutputStream oStream = connection.getOutputStream();
		
		if(proxyHost!=null)
		{
			proxyConnect(oStream);
		}
		
		oStream.write( new String("DELETE "+pageName+"/"+fileName+" HTTP/1.0\r\n").getBytes() );
		addStandardHeader(oStream);
		oStream.write( new String("\r\n").getBytes() );
		oStream.flush();
		
		return connection;
	}	
	
	private void proxyConnect(OutputStream oStream) throws IOException
	{
		oStream.write( new String("CONNECT "+proxyHost+":"+proxyPort+"\r\n").getBytes() );
		oStream.write( new String("HOST "+proxyHost+":"+proxyPort+"\r\n").getBytes() );
	}
	
	private void addStandardHeader(OutputStream oStream) throws IOException
	{
		if(authentication!=null && authentication.length()>0)
		{
			oStream.write( new String("Authorization: "+authentication+"\r\n").getBytes() );
		}
		oStream.write( new String("User-Agent: OpenForum Wiki Applet\r\n").getBytes() );
	}
	
	public void disconnect() throws IOException
	{
		connection.close();
	}
}
