package org.one.stone.soup.xml;

import java.io.*;
import java.util.*;

import org.one.stone.soup.util.*;
import org.one.stone.soup.stringhelper.*;

 /*
 * Wet-Wired.com Xml Version Date 25-Aug-2004
 *
 * Copyright 2000-2001 by Wet-Wired.com Ltd.,
 * Portsmouth England
 * This software is OSI Certified Open Source Software
 * This software is covered by an OSI approved open source licence
 * which can be found at http://www.onestonesoup.org/OSSLicense.html
 */

/**
	* The XmlElement class is the core class in the MXP package.
	* It holds the element name along with the associated attributes and chlid elements.
	* Both attributes and child elements can be hashed to improve searching.
	*
	* This class can also hold debug information like start end end line number and
	* the elements depth in the document tree.
*/

public class XmlElement {

	public static int VALUE = 0;
	public static int ELEMENT = 1;

	public static int UNKNOWN=0;
	public static int SYSTEM_GENERATED=-1;

	private TagTokens tokens = TagTokens.STANDARD_TAGS;

	private String name;
	private Vector attributes = new Vector();
	private Vector data = new Vector();

	private boolean compilerDirective = false;

	private int startLineNo=UNKNOWN;
	private int endLineNo=UNKNOWN;
	private int level=UNKNOWN;
	
	public class Attribute {
		public String name;
		public String separator;
		public String value;
		
		public Attribute() {
			super();
		}
		/**
		 * Attribute constructor comment.
		 */
		public Attribute(String name,String value) {
			super();
	
			this.name=name;
			this.value=value;
		}
		public void parse(String source,String separator)//,AttributeConverter converter)
		{
	
			this.separator = separator;
			int index = source.indexOf(separator);
	
			if(index<1)
				return;
	
			name = source.substring(0,index);
			value = source.substring(index+separator.length());
			if(value.length()>0 && value.charAt(0)=='"')
			{
				value = value.substring(1,value.length()-1);
			}
	
		/*	if(converter!=null)
			{
				nameId=converter.getId(name);
			}*/
		}
		public void setName(String name)
		{
			this.name = name;
		}
		public void setSeparator(String separator)
		{
			this.separator = separator;
		}
		public void setValue(String value)
		{
			this.value = value;
		}
		public String toString() {
			return name+"=\""+value+"\"";
		}
		public boolean isSystemAttribute()
		{
			if(name.charAt(0)=='_')
			{
				return true;
			}
			else
			{
				return false;
			}
		}
	}
	
/**
 * This constructor creates an element with the given name
 * @param name	The name of the element
 */

public XmlElement(String name) {
	this(name,"");
}
/**
 * This constructor creates an element with the given name and records it's start line number.
 * @param name		The elements name
 * @param lineNo	The elements start line number
 */

public XmlElement(String name,int lineNo) {
	this(name,"",lineNo);
}
/**
 * This constructor creates an element with the given name, and adds a text node of value.
 * @param name		The name of the element
 * @param value		The text data contained in the element
 */

public XmlElement(String name,String value) {
	this(name,value,UNKNOWN);
}
/**
 * This constructor creates an element with the given name and a text node of the given value.
 * It also records the start line.
 * @param name		The name of the element
 * @param value		The text data contained in the element
 * @param lineNo	The elements start line number
 */

public XmlElement(String name,String value,int lineNo) {
	super();
	
	if(name==null)
	{
		throw new NullPointerException("XmlElements name cannot be null");
	}
	
	this.name = name;
	startLineNo=lineNo;

	if(value!=null && value.length()>0)
	{
		addValue(value);
	}
}
/**
 * Creates an attribute with name and value and adds this to the element.
 * @param name		The attributes name
 * @param value		The attributes value
 */

public void addAttribute(String name,String value) {

	if(name.charAt(0)=='_')
	{
		if(name.equals("_startLine") && value.length()>0)
		{
			setStartLineNo( Integer.parseInt(value) );
		}
		else if(name.equals("_endLine") && value.length()>0)
		{
			setEndLineNo( Integer.parseInt(value) );
		}
	}
	
	Attribute found = null;
	for(int loop=0;loop<attributes.size();loop++)
	{
		Attribute attribute = (Attribute)attributes.elementAt(loop);
		if(attribute.name.equals(name))
		{
			found= attribute;
			break;
		}
	}
	if(found!=null)
	{
		found.value = value;
		return;
	}

	attributes.addElement( new Attribute(name,value) );
}
/**
 * Removes the attribute with the required name from the element.
 * @param name		The attributes name
 */
public void removeAttribute(String name) {
	attributes.remove( getAttributeByName(name) );
}

/**
 * Adds an attribute to the element.
 * @param attrib		The attribute to add
 */

/*public void addAttribute(Attribute attrib) {
	if(getAttributeValueByName(attrib.name)!=null)
	{
		setAttributeValue(attrib.name,attrib.value);
		return;
	}

	attributes.addElement(attrib);
	attributesHash.put(attrib.name,attrib.value);
}*/
/**
 * Creates a child of this element with the given name.
 * Returns the new child element.
 * @param name		The name of the child element
 */

public XmlElement addChild(String name){

	XmlElement element = new XmlElement(name);
	element.setLevel(level+1);
	data.addElement(element);

	return element;
}
/**
 * Creates a child of this element with the given name and adds a text value to the child element.
 * Returns the new child element.
 * @param name		The name of the child element
 * @param value		The value to be held by the child element
 */

public XmlElement addChild(String name,String value){

	XmlElement element = new XmlElement(name,value);
	element.setLevel(level+1);
	data.addElement(element);

	return element;
}
/**
 * Adds a child element to this element.
 * @param element		The child element to add
 */

public void addChild(XmlElement element){

	if(element==null)
		return;

	element.setLevel(level+1);
	data.addElement(element);
}

public void insertChild(XmlElement element,int index){

	if(element==null)
		return;

	element.setLevel(level+1);
	data.insertElementAt(element,index);
}

protected void addData(Vector newData){

	data.addAll( newData );
}
/**
 * Adds an object as a child of this element
 * @param newData		The vector to be added
 */

public void addDataItem(Object newData){

	//VectorHelper.addVectors( newData,data );
	data.addElement(newData);
}
/*public void addDataItem(String newData){

	//VectorHelper.addVectors( newData,data );
	data.addElement( StringGenerator.convertToHtmlString(newData) );
}*/
/**
 * Adds a text node to this element
 * @param newValue	The text to be added
 */

public void addValue(String newValue) {

	if(newValue==null/* || newValue.length()==0*/)
		return;

	//System.out.println("Added:'"+newValue+"'");

	addDataItem( newValue );
}
/**
 * Checks to see if an attribute with the name provided exists.
 * If it exists and it's value is "true" then true is returned.
 * Else the method returns false
 * @param name		The name of the attribute to check
 */

public boolean attributeIsTrue(String name) {

	String value = getAttributeValueByName(name);
	if(value==null)
	{
		return false;
	}
	else if(value.equals("true"))
	{
		return true;
	}
	else
	{
		return false;
	}
}
/**
 * Returns the data type code for the child node at a given index
 * This returns either XmlElement.VALUE or XmlElement.ELEMENT
 * @param index		The child entry to check
 */

public int dataTypeFor(int index) {
	if( data.elementAt(index) instanceof String )
		return VALUE;
	else return ELEMENT;
}
public String getAttributeNameByIndex(int index){

	if(index>attributes.size())
		return null;

	return ((Attribute)attributes.elementAt(index)).name;
}
public String getAttributeValueByIndex(int index){

	if(index>attributes.size())
		return null;

	return ((Attribute)attributes.elementAt(index)).value;
}
public int getAttributeCount(){

	return attributes.size();
}
/*public Vector getAttributes() {
	return attributes;
}*/
public String getAttributeValueByName(String name){

	Attribute attrib = getAttributeByName(name);

	if(attrib==null)
		return null;
	else
		return (String)attrib.value;
}

private Attribute getAttributeByName(String name){

	Attribute attrib = null;
	for(int loop=0;loop<attributes.size();loop++)
	{
		if( ((Attribute)attributes.elementAt(loop)).name.equals(name) )
		{
			attrib = (Attribute)attributes.elementAt(loop);
		}
	}

	if(attrib==null)
		return null;
	else
		return attrib;
}

public Object getData(int index){

	if(index>data.size())
		return null;

	/*if(data.elementAt(index) instanceof String)
	{
		return StringGenerator.convertHtmlToString(data.elementAt(index).toString());
	}
	else
	{*/
		return data.elementAt(index);
	//}
}
public XmlElement getElementByIndex(int index){

	if(index>=data.size())
		return null;

	if(data.elementAt(index) instanceof XmlElement)
		return (XmlElement)data.elementAt(index);

	return null;
}
public int getElementIndex(XmlElement node)
{
	return data.indexOf(node);
}

public XmlElement getElementByName(String name){

	XmlElement element = null;
	for(int loop=0;loop<data.size();loop++)
	{
		Object item = data.elementAt(loop);
		if(item instanceof XmlElement)
		{
			if( ((XmlElement)item).getName().equals(name) )
			{
				element = (XmlElement)item;
			}
		}
	}

	if(element==null)
		return null;
	else
		return (XmlElement)element;

/*
	XmlElement element;
	for(int loop=0;loop<data.size();loop++)
	{
		if(data.elementAt(loop) instanceof XmlElement)
		{
			element = (XmlElement)data.elementAt(loop);
			if(element.name.equals(name))
				return element;
		}
	}

	return null;*/
}
public int getElementCount(){
	return data.size();
}
public Vector getElements(){
	return data;
}
public Vector getElementsByName(String name){

	Vector elements = new Vector();
	XmlElement element;
	for(int loop=0;loop<data.size();loop++)
	{
		if(data.elementAt(loop) instanceof XmlElement)
		{
			element = (XmlElement)data.elementAt(loop);
			if(element.getName().equals(name))
				elements.addElement(element);
		}
	}

	return elements;
}
/**
 * Insert the method's description here.
 * Creation date: (19/07/00 17:56:33)
 * @return int
 */
public int getEndLineNo() {
	return endLineNo;
}
/**
 * Insert the method's description here.
 * Creation date: (19/07/00 17:56:33)
 * @return int
 */
public int getLevel() {
	return level;
}
public String getName() {
	return name;
}
/**
 * Insert the method's description here.
 * Creation date: (19/07/00 17:56:33)
 * @return int
 */
public int getStartLineNo() {
	return startLineNo;
}
/**
 *
 * @return org.one.stone.soup.xml.TagTokens
 */
public TagTokens getTagTokens() {
	return tokens;
}
public String getValue() {

	if(data.size()>0 && data.elementAt(0) instanceof String)
	{
		return data.elementAt(0).toString();
		//return StringGenerator.convertHtmlToString(data.elementAt(0).toString());
	}
	else
		return "";
}
public String getValue(int index) {

	if(data.size()<=index)
		return null;

	if(data.elementAt(index) instanceof String)
	{
		return data.elementAt(index).toString();
		//return StringGenerator.convertHtmlToString(data.elementAt(index).toString());
	}
	else
		return null;
}
public int getValueSize() {

	return data.size();
}
public boolean hasChild(String name) {

	if(getElementByName(name)==null)
		return false;
	else
		return true;
}
/**
 *
 * @return boolean
 */
public boolean isCompilerDirective() {
	return compilerDirective;
}

public void removeChild( XmlElement child ){

	data.removeElement( child );

}
public void removeData( int index ){

	Object child = getData(index);
	if(child instanceof XmlElement)
	{
		removeChild((XmlElement)child);
	}
	else
	{
		data.removeElementAt( index );
	}
}
public void removeData( Object newData ){

	data.removeElement( newData );

}
public void setAttributeValue(String name,String value)
{
	getAttributeByName(name).setValue(value);
}
/**
 *
 * @param newCompilerDirective boolean
 */
public void setCompilerDirective(boolean newCompilerDirective) {
	compilerDirective = newCompilerDirective;
}
public void setData(int index,Object newData){

	/*if(newData instanceof String)
	{
		newData = StringGenerator.convertToHtmlString(newData.toString());
	}*/
	data.setElementAt(newData,index);
}
/**
 * Insert the method's description here.
 * Creation date: (19/07/00 17:56:33)
 * @param newLineNo int
 */
public void setEndLineNo(int newLineNo) {
	endLineNo = newLineNo;
}
/**
 * Insert the method's description here.
 * Creation date: (19/07/00 17:56:33)
 * @param newLevel int
 */
public void setLevel(int newLevel) {

	if(/*level!=newLevel && */data.size()>0)
	{
		for(int loop=0;loop<data.size();loop++)
		{
			if(data.elementAt(loop) instanceof XmlElement)
				((XmlElement)data.elementAt(loop)).setLevel(newLevel+1);
		}
	}

	level = newLevel;
}
public void setName(String newName) {
	name = newName;
}
/**
 * Insert the method's description here.
 * Creation date: (19/07/00 17:56:33)
 * @param newLineNo int
 */
public void setStartLineNo(int newLineNo) {
	startLineNo = newLineNo;
}
public void setTagTokens(TagTokens newTokens) {
	tokens = newTokens;
}

public boolean setValue(String newValue) {
	return setValue(newValue,0);
}

public boolean setValue(String newValue,int index) {

	/*if(newValue.length()==0)
		return false;*/

	if(data.size()<index)
		return false;

	if(data.size()==index)
	{
		addValue(newValue);
	}

/*	if(data.elementAt(index) instanceof String)
	{*/
		data.setElementAt( newValue,index );
		//data.setElementAt( StringGenerator.convertToHtmlString(newValue),index);
		return true;
/*	}
	else
		return false;*/
}
public String toString() {
	String string=name;

	Attribute attribute;
	for(int loop=0;loop<attributes.size();loop++)
	{
		attribute = (Attribute)attributes.elementAt(loop);

		//if( attribute.name.charAt(0)=='_' )
		//	continue;

		string+=" "+attribute.name+"=\""+attribute.value+"\"";
	}

	return string;
}
public String toXml() {
	return toXml(true);
}
public String toXml(boolean beautify) {
	if(beautify)
		return toXmlBeautify();
	else
		return toXmlNoBeautify();
}
public String toXml(boolean beautify,boolean ignorSystemAttributes) {
	if(beautify)
		return toXmlBeautify(ignorSystemAttributes);
	else
		return toXmlNoBeautify(ignorSystemAttributes);
}
public String toXmlBeautify() {
	return toXmlBeautify(true);
}
public String toXmlBeautify(boolean ignorSystemAttributes) {

	// Create a padding string of tabs for the current elements level
	String pad=StringGenerator.generatePad(level,'\t');
	/*if(level>0)
	{
		pad=TextConstants.TEXT_CRLF+pad;		
	}*/

	// If the element is a compiler directive ie. <?...?>
	if( isCompilerDirective() )
	{
		// Return a compiler directive string

		String string=pad+tokens.getStartTag()+"?"+name;

		Attribute attribute;
		for(int loop=0;loop<attributes.size();loop++)
		{
			attribute = (Attribute)attributes.elementAt(loop);

			if( ignorSystemAttributes==true && attribute.name.charAt(0)=='_' )
				continue;

			string+=" "+attribute.name+"=\""+attribute.value+"\"";
		}
		string+="?"+tokens.getCloseTag()+TextConstants.TEXT_CRLF;

		return string;
	}

	// Create the opening element string ie. "(padding)<name"
	String string=pad+tokens.getStartTag()+name;

	// Add each of the attributes to the string ie. "(padding)<name attribA="data" attribB="data"..."
	Attribute attribute;
	for(int loop=0;loop<attributes.size();loop++)
	{
		attribute = (Attribute)attributes.elementAt(loop);

		if( ignorSystemAttributes==true && attribute.isSystemAttribute() )
		{
			continue;
		}

		string+=" "+attribute.name+"=\""+attribute.value+"\"";
	}

	if( ignorSystemAttributes==false )
	{
		string+=" _startLine=\""+getStartLineNo()+"\"";
		string+=" _endLine=\""+getEndLineNo()+"\"";
	}	
	
	// If the element has no children, close the element ie. "(padding)<name attribA="data" attribB="data".../>"
	if(data.size()==0)
	{
		string+=tokens.getCloseEmptyTag()+TextConstants.TEXT_CRLF;
	}
	else
	{
		// Close the tag ie. "(padding)<name attribA="data" attribB="data"...>"
		string+=tokens.getCloseTag();//+TextConstants.TEXT_CRLF;

		// Add the elements children
		if(data.size()>0)
		{
			XmlElement element;
			int valueCount = 0;
			int elementCount = 0;
			for(int loop=0;loop<data.size();loop++)
			{
				// If the child is another element
				if(data.elementAt(loop) instanceof XmlElement)
				{
					// Call the same method for the child and add the returned string
					element = (XmlElement)data.elementAt(loop);
					if(loop==0)
					{
						string+=TextConstants.TEXT_CRLF;
					}
					string+=element.toXmlBeautify( ignorSystemAttributes );
					elementCount++;
				}
				// If the child is a vector
				else if(data.elementAt(loop) instanceof Vector)
				{
					// Add the result of toString() for each item in the vector
					string+="<![CDATA[";
					
					Vector v= (Vector)data.elementAt(loop);
					for(int loopV=0;loopV<v.size();loopV++)
					{
						string+=v.elementAt(loopV)+TextConstants.TEXT_CRLF;
						valueCount++;
					}
					
					string+="]]>";//+TextConstants.TEXT_CRLF;
				}
				// Else
				else
				{
					String value = data.elementAt(loop).toString();
					if(
							value.indexOf('\n')>-1 ||
							value.indexOf('<')>-1 ||
							value.indexOf('>')>-1 ||
							value.indexOf('/')>-1
					)
					{
						string+="<![CDATA["+value+"]]>";
						valueCount++;
					}
					else
					{
						string+=value;
						valueCount++;
					}
				}
			}
			if( (valueCount==1 && elementCount==0)==false )
			{
				string+=pad;
			}
		}
		// Close the element ie. <name ...>...</name>
		string+=tokens.getEndTag()+name+tokens.getCloseTag();
		
		if(level>0)
		{
			string = string+TextConstants.TEXT_CRLF;
		}
	}

	return string;
}
public String toXmlNoBeautify() {
	return toXmlNoBeautify(true);
}
public String toXmlNoBeautify(boolean ignorSystemAttributes) {

	if( isCompilerDirective() )
	{
		String string=tokens.getStartTag()+"?"+name;

		Attribute attribute;
		for(int loop=0;loop<attributes.size();loop++)
		{
			attribute = (Attribute)attributes.elementAt(loop);

			if( ignorSystemAttributes==true && attribute.isSystemAttribute() )
				continue;

			string+=" "+attribute.name+"=\""+attribute.value+"\"";
		}
		string+="?"+tokens.getCloseTag();

		return string;
	}

	String string=tokens.getStartTag()+name;

	Attribute attribute;
	for(int loop=0;loop<attributes.size();loop++)
	{
		attribute = (Attribute)attributes.elementAt(loop);

		if( ignorSystemAttributes==true && attribute.isSystemAttribute() )
			continue;

		string+=" "+attribute.name+"=\""+attribute.value+"\"";
	}
	string+=" _startLine=\""+getStartLineNo()+"\"";
	string+=" _endLine=\""+getEndLineNo()+"\"";
	
/*	if(data.size()==0)
	{
		string+=tokens.getCloseEmptyTag();
	}
	else
	{*/
		string+=tokens.getCloseTag();
		if(data.size()>0)
		{
			//boolean inQuotes = false;
			XmlElement element;
			for(int loop=0;loop<data.size();loop++)
			{
				if(data.elementAt(loop) instanceof XmlElement)
				{
					/*if(inQuotes==true)
					{
						string+="\"";
						inQuotes = false;
					}*/
					element = (XmlElement)data.elementAt(loop);
					string+=element.toXmlNoBeautify( ignorSystemAttributes );
				}
				// If the child is a vector
				else if(data.elementAt(loop) instanceof Vector)
				{
					/*if(inQuotes==false)
					{
						string+="\"";
						inQuotes = true;
					}*/
					// Add the result of toString() for each item in the vector
					string+="<![CDATA[";
					
					Vector v= (Vector)data.elementAt(loop);
					for(int loopV=0;loopV<v.size();loopV++)
					{
						string+=v.elementAt(loopV);
					}
					
					string+="]]>";
				}
				// Else
				else
				{
					/*if(inQuotes==false)
					{
						string+="\"";
						inQuotes = true;
					}*/
					// Add the result of toString for the data
					//string+=pad+data.elementAt(loop)+TextConstants.TEXT_CRLF;
					String value = data.elementAt(loop).toString();
					if(
							value.indexOf('\n')>-1 ||
							value.indexOf('<')>-1 ||
							value.indexOf('>')>-1 ||
							value.indexOf('/')>-1
					)
					{
						string+="<![CDATA["+value+"]]>";
					}
					else
					{
						string+=value;					
					}
				}
			}
			/*if(inQuotes==true)
			{
				string+="\"";
				inQuotes = false;
			}*/
		}
		string+=tokens.getEndTag()+name+tokens.getCloseTag();
/*	}*/

	return string;
}

public void toXmlBeautify(OutputStream oStream,boolean ignorSystemAttributes) throws IOException {

	// Create a padding string of tabs for the current elements level
	String pad=StringGenerator.generatePad(level,'\t');

	// If the element is a compiler directive ie. <?...?>
	if( isCompilerDirective() )
	{
		// Return a compiler directive string

		oStream.write( (pad+tokens.getStartTag()+"?"+name).getBytes() );

		Attribute attribute;
		for(int loop=0;loop<attributes.size();loop++)
		{
			attribute = (Attribute)attributes.elementAt(loop);

			if( ignorSystemAttributes==true && attribute.name.charAt(0)=='_' )
				continue;

			oStream.write( (" "+attribute.name+"=\""+attribute.value+"\"").getBytes() );
		}
		oStream.write( ("?"+tokens.getCloseTag()+TextConstants.TEXT_CRLF).getBytes() );

		return;
	}

	// Create the opening element string ie. "(padding)<name"
	oStream.write( (pad+tokens.getStartTag()+name).getBytes() );

	// Add each of the attributes to the string ie. "(padding)<name attribA="data" attribB="data"..."
	Attribute attribute;
	for(int loop=0;loop<attributes.size();loop++)
	{
		attribute = (Attribute)attributes.elementAt(loop);

		if( ignorSystemAttributes==true && attribute.isSystemAttribute() )
			continue;

		oStream.write( (" "+attribute.name+"=\""+attribute.value+"\"").getBytes() );
	}

	// If the element has no children, close the element ie. "(padding)<name attribA="data" attribB="data".../>"
	if(data.size()==0)
	{
		oStream.write( (tokens.getCloseEmptyTag()+TextConstants.TEXT_CRLF).getBytes() );
	}
	else
	{
		// Close the tag ie. "(padding)<name attribA="data" attribB="data"...>"
		oStream.write( (tokens.getCloseTag()+TextConstants.TEXT_CRLF).getBytes() );

		// Add the elements children
		if(data.size()>0)
		{
			//boolean inQuotes = false;
			XmlElement element;
			for(int loop=0;loop<data.size();loop++)
			{
				// If the child is another element
				if(data.elementAt(loop) instanceof XmlElement)
				{
					/*if(inQuotes==true)
					{
						string+="\"";
						inQuotes = false;
					}*/
					// Call the same method for the child and add the returned string
					element = (XmlElement)data.elementAt(loop);
					element.toXmlBeautify( oStream,ignorSystemAttributes );
				}
				// If the child is a vector
				else if(data.elementAt(loop) instanceof Vector)
				{
					/*if(inQuotes==false)
					{
						string+="\"";
						inQuotes = true;
					}*/
					// Add the result of toString() for each item in the vector
					Vector v= (Vector)data.elementAt(loop);
					for(int loopV=0;loopV<v.size();loopV++)
					{
						oStream.write( (pad+v.elementAt(loopV)+TextConstants.TEXT_CRLF).getBytes() );
					}
				}
				// Else
				else
				{
					/*if(inQuotes==false)
					{
						string+="\"";
						inQuotes = true;
					}*/
					// Add the result of toString for the data
					oStream.write( (pad+data.elementAt(loop)+TextConstants.TEXT_CRLF).getBytes() );
				}
			}
			/*if(inQuotes==true)
			{
				string+="\"";
				inQuotes = false;
			}*/
		}
		// Close the element ie. <name ...>...</name>
		oStream.write( (pad+tokens.getEndTag()+name+tokens.getCloseTag()+TextConstants.TEXT_CRLF).getBytes() );
	}
	oStream.flush();
}

}
