package org.one.stone.soup.xml;

import java.io.*;
import java.util.*;

import org.one.stone.soup.stringhelper.*;



/*
 * Wet-Wired.com Xml Version Date 25-Aug-2004
 *
 * Copyright 2000-2001 by Wet-Wired.com Ltd.,
 * Portsmouth England
 * This software is OSI Certified Open Source Software
 * This software is covered by an OSI approved open source licence
 * which can be found at http://www.onestonesoup.org/OSSLicense.html
 */

/**
 * @author nikcross
 *
 */

public class XmlParser {

	private static boolean traceOn = false;

	private XmlElement ref = new XmlElement("REF");

	private String currentFileName="";

	private int lineNo=0;

	private String value;
	private Vector valueVector;
	private int valueLineNo;

	private String rawTag;
	private int tagLineNo;

	private Stack stack = new Stack();

	private FileOutputStream trace;

	public class XmlCDATAParser implements XmlTagParser {

		public boolean startMatch(StringBuffer data)
		{
			if(data.length()==8 && data.toString().equals("![CDATA["))
			{
				return true;
			}
			return false;
		}
		
		public boolean parseTag(XmlParser parser, String rawTag, XmlDocument doc,
				XmlElement currentElement, InputStream iStream)
				throws XmlParseException {
			
			if( rawTag.length()>=8 && rawTag.substring(0,8).equals("![CDATA[") )
			{	
				StringBuffer buffer = new StringBuffer( rawTag.substring(8) );
				boolean endFound=false;
				
				try{
					int in = iStream.read();
					while(in !=-1)
					{
						char c = (char)in;
						
						switch(c)
						{
							case ']':
								int inA = iStream.read();
								int inB = iStream.read();
								
								while(inA==']' && inB==']')
								{
									buffer.append((char)in);
									in = inA;
									inA = inB;
									inB = iStream.read();
								}
								
								if( inA==']' && inB=='>' )
								{
									endFound=true;
									break;
								}
								else
								{
									buffer.append((char)in);
									buffer.append((char)inA);
									buffer.append((char)inB);
									break;
								}
							default:
								buffer.append( c );
						}
						
						if(endFound==true)
						{
							break;
						}
						in = iStream.read();
					}
					currentElement.addValue( buffer.toString() );
				}
				catch(Exception e)
				{
					throw new XmlParseException(-1,-1);
				}
				
				return true;
			}
			
			return false;
		}

	}

	public class XmlImportParser implements XmlTagParser {

		public boolean startMatch(StringBuffer data)
		{
			if(data.length()==6 && data.toString().equals("import"))
			{
				return true;
			}
			return false;
		}
		
		public boolean parseTag(XmlParser currentParser, String rawTag, XmlDocument doc,
				XmlElement currentElement, InputStream currentIStream)
		throws XmlParseException
		{
			try{
				boolean insideQuotes = false;
				int in = currentIStream.read();
				while(in!=-1 && (in=='/' && insideQuotes==false)==false)
				{
					if(in=='\n')
					{
						currentParser.lineNo++;
					}
					else
					{
						rawTag+=(char)in;
					}
					
					if(in=='\"')
					{
						insideQuotes = !insideQuotes;
					}
					
					in = currentIStream.read();
				}
				in = currentIStream.read();
			}
			catch(IOException e)
			{
				throw new XmlParseException(XmlParseException.UNEXPECTED_END_OF_TAG,currentParser.lineNo);
			}
			
			XmlElement directive = currentParser.createElement(rawTag,-1);
			
			String fileName = directive.getAttributeValueByName("url");

			try{
				XmlParser parser =  new XmlParser();
				XmlElement newElement = null;
				
				if( fileName != null )
				{
					InputStream iStream = null;
					iStream = org.one.stone.soup.browser.Browser.getInputStream(fileName);
					parser.setCurrentFileName( org.one.stone.soup.browser.Browser.getURLFor(fileName) );
					newElement = parser.parseDocument(iStream);
					iStream.close();
				}
				else
				{
					fileName = directive.getAttributeValueByName("file");
					if(fileName==null)
					{
						throw new XmlParseException(
							XmlParseException.NO_FILE_TO_IMPORT,
							new Object[]{
								""+lineNo,
								"in file "+currentFileName
							}
						);
					}

					File file = new File( new File(currentFileName).getParentFile().getAbsolutePath()+"/"+fileName );
					newElement = parser.parseDocument(file);
				}

				// strip off document wrapper
				for(int loop2=0;loop2<newElement.getElementCount();loop2++)
				{
					if(newElement.dataTypeFor(loop2)==XmlElement.VALUE)
						currentElement.addValue(newElement.getValue(loop2));
					else
						currentElement.addChild(newElement.getElementByIndex(loop2));
				}
			}
			catch(IOException e)
			{
				throw new XmlParseException(
					XmlParseException.FAILED_TO_IMPORT_FILE,
					new Object[]{
						fileName,
						""+lineNo,
						e.toString()+" "+e.getMessage() + "in file" + currentFileName
					}
				);
			}

			return true;
		}
	}
	
	public class XmlCommentParser implements XmlTagParser {

		public boolean startMatch(StringBuffer data)
		{
			if(data.length()==3 && data.toString().equals("!--"))
			{
				return true;
			}
			return false;
		}
		
		public boolean parseTag(XmlParser parser, String rawTag, XmlDocument doc,
				XmlElement currentElement, InputStream iStream)
				throws XmlParseException {
			
			if( rawTag.length()>=3 && rawTag.substring(0,3).equals("!--") )
			{	
				StringBuffer buffer = new StringBuffer( rawTag.substring(3) );
				boolean endFound=false;
				
				try{
					int in = iStream.read();
					while(in !=-1)
					{
						char c = (char)in;
						
						switch(c)
						{
							case '-':
								int inA = iStream.read();
								int inB = iStream.read();
								if( inA=='-' && inB=='>' )
								{
									endFound=true;
									break;
								}
								else
								{
									buffer.append((char)in);
									buffer.append((char)inA);
									buffer.append((char)inB);
									break;
								}
							default:
								buffer.append( c );
						}
						
						if(endFound==true)
						{
							break;
						}
						in = iStream.read();
					}
					//currentElement.addValue( buffer.toString() );
					
					  //System.out.println("Comment:"+buffer.toString());
				}
				catch(Exception e)
				{
					throw new XmlParseException(-1,-1);
				}
				
				return true;
			}
			
			return false;
		}

	}
	

	private XmlCDATAParser cDataParser = new XmlCDATAParser();
	private XmlCommentParser commentParser = new XmlCommentParser();
	private XmlImportParser importParser = new XmlImportParser();	
	
	private Vector compilers = null;
	private Vector tagParsers = null;
	private String[] emptyTags = null;
public XmlParser()
{
	ref.setTagTokens( new TagTokens() );
	tagParsers = new Vector();
	tagParsers.addElement( cDataParser );
	tagParsers.addElement( commentParser );
	tagParsers.addElement( importParser );
	
}

public void removeImportParser()
{
	tagParsers.remove(importParser);
}

public void addCompiler(XmlProcessor compiler)
{
	if(compilers==null)
	{
		compilers = new Vector();
	}

	compilers.addElement(compiler);
}
public void addTagParser(XmlTagParser tagParser)
{
	if(tagParsers==null)
	{
		tagParsers = new Vector();
	}

	tagParsers.addElement(tagParser);
}
/**
 *
 * @return java.lang.String
 */
public String getCurrentFileName() {
	return currentFileName;
}
/**
 *
 * @return java.lang.String[]
 */
public java.lang.String[] getEmptyTags() {
	return emptyTags;
}
/**
 *
 * @return org.one.stone.soup.xml.XmlElement
 */
public XmlElement getReferenceElement() {
	return ref;
}
/**
 *
 * @return boolean
 */
public static boolean isTraceOn() {
	return traceOn;
}
public XmlDocument parseDocument(InputStream iStream) throws XmlParseException{

	return (XmlDocument)parseElement(null,iStream,true);
}
public XmlDocument parseDocument(File file) throws XmlParseException{
	XmlDocument doc=null;
	try{
		setCurrentFileName( file.getAbsolutePath() );
		
		FileInputStream iStream = new FileInputStream(file.getAbsolutePath());
		doc = this.parseDocument(iStream);
		iStream.close();
		
		return doc;
	}
	catch(FileNotFoundException fnf)
	{
		throw new XmlParseException(XmlParseException.FAILED_TO_FIND_FILE,new Object[]{file.getAbsolutePath()});
	}
	catch(IOException ioe)
	{
		return doc;
	}
}
public XmlDocument parseDocument(String data) throws XmlParseException{
	return this.parseDocument(new StringInputStream(data));
}
public XmlElement parseElement(InputStream iStream) throws XmlParseException{
	try{
		return parseElement(null,iStream,false);
	}catch(XmlParseException e)
	{
		try{
			iStream.close();
		}
		catch(Exception e2){}

		throw e;
	}
}
private XmlElement parseElement(XmlElement root,InputStream iStream,boolean isDocument) throws XmlParseException{

	if(traceOn)
	{
		try{
			trace = new FileOutputStream("XmlParser."+StringGenerator.generateUniqueId()+".trace");
		}catch(IOException e){}
	}

	lineNo=1;

	XmlElement currentElement=null;
	XmlElement newElement=null;

	boolean atRoot=true;
	XmlDocument doc = null;

	boolean finished;
	finished = readToNextTag(iStream);

	while(!finished){

		finished = readTag(iStream);

	if(rawTag.length()<1)
	{
		String lineSt = ""+lineNo;
		if(currentFileName!=null && currentFileName.length()>0)
		{
			lineSt+=" in file "+currentFileName;
		}
				throw new XmlParseException(
					XmlParseException.EMPTY_TAG,
					new Object[]{
						lineSt
					}
				);
	}

	  if(rawTag.length()>=1 && rawTag.charAt(0)=='?')//(rawTag.charAt(0)=='?' || rawTag.charAt(0)=='!'))
	  {
		  //System.out.println("XmlCompiler directive:"+rawTag);
		  XmlElement directive = createElement(rawTag.substring(1,rawTag.length()-2),lineNo);//new XmlElement( rawTag.substring(1,rawTag.length()-2),lineNo );
		  directive.setEndLineNo(lineNo);

		if(compilers!=null)
		{
			if(root==null)
			{
				root = doc;
			}
			for(int loop=0;loop<compilers.size();loop++)
			{
				if(
					((XmlProcessor)compilers.elementAt(loop)).process(root,directive,currentElement,lineNo,currentFileName)==true
				)
					break;
			}
		}

	  }
	  /*else if(settings.xmlComplient==true && rawTag.length()>=1 && rawTag.substring(0,1).equals("!") )
	  {
		  System.out.println("Not implemented:"+rawTag);
	  }*/
	  else
	  {
		if( rawTag.charAt(0)=='/' ) // Is end tag
		{
			rawTag = rawTag.substring(1);

			if(rawTag.equals(currentElement.getName()))
			{
				currentElement.setEndLineNo(lineNo);
				if( !stack.isEmpty() ) // Pop higher off stack as current
				{
					currentElement = (XmlElement)stack.pop();
				}
				else
				{
					finished = true;
					break;
				}

			}
			else
			{
				String lineSt = ""+lineNo;
				if(currentFileName!=null && currentFileName.length()>0)
				{
					lineSt+=" in file "+currentFileName;
				}

					throw new XmlParseException(
						XmlParseException.END_TAG_MISMATCH,
						new Object[]{
							lineSt,
							rawTag,
							currentElement.getName(),
							""+currentElement.getStartLineNo()
						}
					);
			}
		}
		else										// Is start tag
		{
			if(emptyTags!=null)
			{
				XmlElement testElement = createElement( rawTag,lineNo);
				for(int loop=0;loop<emptyTags.length;loop++)
				{
					if(testElement.getName().equals(emptyTags[loop] ))
					{
						rawTag+="/";
					}
				}
			}

			if(rawTag.charAt(rawTag.length()-1)=='/')// Is empty tag
			{
				newElement = createElement( rawTag.substring(0,rawTag.length()-1),lineNo);
				/*if(currentFileName!=null && currentFileName.length()>0)
				{
					newElement.addAttribute("_src",currentFileName);
				}*/
				newElement.setEndLineNo(lineNo);
				if(currentElement!=null)
					currentElement.addChild(newElement);
				else
				{
					if(atRoot && isDocument) // Document is empty
					{
						doc=createDocument( rawTag.substring(0,rawTag.length()-1),XmlElement.UNKNOWN );

						if(root==null)
						{
							root = doc;
						}
						return doc;
					}

					if(root==null)
					{
						root = newElement;
					}
					return newElement; //only one empty element found
				}
			}
			else									// Not empty tag
			{
				boolean subParsed = false;

				if(tagParsers!=null)
				{
					for(int loop=0;loop<tagParsers.size();loop++)
					{
						XmlTagParser tagParser = (XmlTagParser)(tagParsers.elementAt(loop));
						if( tagParser.startMatch(new StringBuffer(rawTag))==true && tagParser.parseTag(this,rawTag,doc,currentElement,iStream)==true )
						{
							subParsed=true;
							break;
						}
					}
				}

				if(subParsed==false)
				{
					if(atRoot && isDocument)
					{
						doc=createDocument(rawTag,XmlElement.UNKNOWN);
						newElement = doc;
						atRoot=false;
					}
					else
						newElement = createElement(rawTag,lineNo);

					if(currentElement!=null)
					{
						currentElement.addChild(newElement);
						stack.push(currentElement);
					}

					currentElement=newElement;
				}
			}
		}
	  }

	finished = readToNextTag(iStream);

		if(currentElement!=null)
		{
			if(valueVector==null)
			{
				//value=value.trim();
				if(value.length()>0)
				{
					currentElement.addValue(value);
				}
			}
			else
			{
				currentElement.addData(valueVector);
			}
		}
	}

	if(traceOn)
	{
		try{
			trace.close();
		}catch(IOException ie){};
	}

	if(isDocument)
	{
		if(root==null)
		{
			root = doc;
		}
		return doc;
	}
	else
	{
		if(root==null)
		{
			root = currentElement;
		}
		return currentElement;
	}
}

public XmlElement parseElement(File file) throws XmlParseException{
	XmlElement element=null;
	try{
		setCurrentFileName( file.getAbsolutePath() );
		
		FileInputStream iStream = new FileInputStream(file);
		element = this.parseElement(iStream);
		iStream.close();
		
		return element;
	}
	catch(FileNotFoundException fnf)
	{
		throw new XmlParseException(XmlParseException.FAILED_TO_FIND_FILE,new Object[]{file.getAbsolutePath()});
	}
	catch(IOException ioe)
	{
		return element;
	}
}
public XmlElement parseElement(String data) throws XmlParseException{
	return this.parseElement(new StringInputStream(data));
}
private boolean readTag(InputStream iStream) throws XmlParseException {

	StringBuffer buffer = new StringBuffer();
	tagLineNo=lineNo;
	boolean inQuotes = false;

	try{

	int in=iStream.read();
	if(traceOn)
	{
		trace.write((char)in);
	}
	//System.out.println(in+",");
	while(in!=-1 && (inQuotes || (in!=ref.getTagTokens().getEndTagChar())) )
	{
		if(in=='\"')
		{
			inQuotes=!inQuotes;
		}
		if(in<0x20)
		{
			if(in==0x0D)
				lineNo++;

			in=' ';
			//in=iStream.read();
			//continue;
		}

		//System.out.println(in+",");
		buffer.append( (char)in );
		
		for(int loop=0;loop<tagParsers.size();loop++)
		{
			XmlTagParser tagParser = (XmlTagParser)tagParsers.elementAt(loop);
			if(tagParser.startMatch(buffer))
			{
				rawTag=buffer.toString();
				return true;
			}
		}
		
		in=iStream.read();
		
		if(traceOn)
		{
			trace.write((char)in);
		}
	}

	rawTag = buffer.toString().trim();

//	System.out.println("Raw tag '"+rawTag+"'");

	if(in==-1)
		return true;
	else
		return false;

	}
	catch(IOException e)
	{
		throw new XmlParseException(
			XmlParseException.UNEXPECTED_END_OF_TAG,
			new Object[]{
				buffer.toString(),
				""+lineNo,
				e.getMessage()
			}
		);
	}
}

private boolean readToNextTag(InputStream iStream) throws XmlParseException{


	StringBuffer buffer = new StringBuffer();
	valueLineNo=lineNo;
	valueVector=null;

 try{

	int in=iStream.read();

	while(in!=-1 && in!=ref.getTagTokens().getStartTagChar())
	{
		if(in==0x0A)
		{
			lineNo++;
		}

		buffer.append((char)in);

		in=iStream.read();
		if(traceOn)
		{
			trace.write((char)in);
		}
	}

	value = buffer.toString().trim();

	if(in==-1)
		return true;
	else
		return false;

 }
 catch(IOException e)
 {
		throw new XmlParseException(
			XmlParseException.UNEXPECTED_END_OF_TAG,
			new Object[]{
				buffer.toString(),
				""+lineNo,
				e.getMessage()
			}
		);
 }
}
/**
 *
 * @param newCurrentFileName java.lang.String
 */
public void setCurrentFileName(String newCurrentFileName) {
	currentFileName = newCurrentFileName;
}
/**
 *
 * @param newEmptyTags java.lang.String[]
 */
public void setEmptyTags(java.lang.String[] newEmptyTags) {
	emptyTags = newEmptyTags;
}

/**
 *
 * @param newTraceOn boolean
 */
public static void setTraceOn(boolean newTraceOn) {
	traceOn = newTraceOn;
}
/**
 *
 * @return java.lang.String
 */
public String toString() {

	String state = super.toString()+" Parsing at line "+lineNo+"\n";
	state+=" value=\""+value+"\" line:"+valueLineNo+"\n";
	state+=" raw tag="+rawTag+" line:"+tagLineNo+"\n";
/*	if(MODE==LOCAL_MODE)
		state+=" mode=LOCAL\n";
	else if(MODE==REMOTE_MODE)
		state+=" mode=REMOTE\n";*/
	if(currentFileName.length()>0)
		state+=" file:"+currentFileName+"\n";
	state+=" stack height="+stack.size()+"\n";

	return state;
}
private XmlDocument createDocument( String data,int lineNumber)
{
	XmlDocument newDocument =  new XmlDocument("_not-set");
	parseNode( newDocument,data );
	
	newDocument.setStartLineNo(lineNumber);
	if(newDocument.getAttributeValueByName("_src")==null && currentFileName!=null && currentFileName.length()>0)
	{
		newDocument.addAttribute("_src",currentFileName);
	}	
	
	return newDocument;
}
private XmlElement createElement( String data,int lineNumber)
{
	XmlElement newElement =  new XmlElement("_not-set");
	parseNode( newElement,data );

	newElement.setStartLineNo(lineNumber);
	if(newElement.getAttributeValueByName("_src")==null && currentFileName!=null && currentFileName.length()>0)
	{
		newElement.addAttribute("_src",currentFileName);
	}
	
	return newElement;
}

private void parseNode(XmlElement element,String data)
{
	int startCursor = 0;
	int endCursor = 0;

	//Read past space
	while(startCursor<data.length() && Character.isWhitespace(data.charAt(startCursor)))
	{
		startCursor++;
	}
	endCursor = startCursor;
	
	// Read node name
	while(endCursor<data.length() && !Character.isWhitespace(data.charAt(endCursor)))
	{
		endCursor++;
	}
	element.setName( data.substring(startCursor,endCursor) );
	startCursor = endCursor;
	
	//Read past space
	while(startCursor<data.length() && Character.isWhitespace(data.charAt(startCursor)))
	{
		startCursor++;
	}
	endCursor = startCursor;
	
	while(endCursor<data.length())
	{
		// Read attribute name
		while(endCursor<data.length() && !(Character.isWhitespace(data.charAt(endCursor)) || data.charAt(endCursor)=='='))
		{
			endCursor++;
		}
		String attributeName = data.substring(startCursor,endCursor);
		startCursor = endCursor;
		
		//Read past space and =
		while(startCursor<data.length() && (Character.isWhitespace(data.charAt(startCursor)) || data.charAt(startCursor)=='='))
		{
			startCursor++;
		}
		endCursor = startCursor;
		
		// Read attribute value
		boolean inQuotes = false;
		char end = ' ';
		if(startCursor<data.length() && (data.charAt(startCursor)=='\'' || data.charAt(startCursor)=='"'))
		{
			end = data.charAt(startCursor);
			inQuotes = true;
			startCursor++;
			endCursor = startCursor;
		}
		while(endCursor<data.length())
		{
			if(inQuotes)
			{
				if(data.charAt(endCursor)==end)
				{
					break;
				}
			}
			else if(Character.isWhitespace(data.charAt(endCursor)))
			{
				break;
			}
			endCursor++;
		}
		String attributeValue = data.substring(startCursor,endCursor);
		if(inQuotes)
		{
			endCursor++;
		}
		startCursor = endCursor;
		
		element.addAttribute(attributeName,attributeValue);

		
		//Read past space
		while(startCursor<data.length() && Character.isWhitespace(data.charAt(startCursor)))
		{
			startCursor++;
		}
		endCursor = startCursor;
	}
}
}
