package test.org.one.stone.soup.xml.helper;

import org.one.stone.soup.xml.XmlElement;
import org.one.stone.soup.xml.helper.XmlPathSearch;

import junit.framework.TestCase;

public class XmlPathSearchTests extends TestCase {
	
	private XmlElement testDocument;
	
	public void setUp()
	{
		testDocument = new XmlElement("root");
		testDocument.addChild("child").addChild("subchild").addAttribute("name","nameA");
		testDocument.addChild("child").addChild("subchild").addAttribute("name","nameB");
		testDocument.addChild("child").addChild("subchild").addAttribute("name","nameC");
	}
	
	public void testSearchByAttribute()
	{
		XmlPathSearch search = new XmlPathSearch(testDocument);
		
		XmlElement result = search.findElement("root/child/subchild[@name='nameB']");
		
		assertNotNull( result );
		assertEquals( result.getName(),"subchild" );
		assertEquals( result.getAttributeValueByName("name"),"nameB" );
		
	}
	
	public void testSearchInnerQuery()
	{
		XmlPathSearch search = new XmlPathSearch(testDocument);
		
		XmlElement result = search.findElement("root/child[subchild[@name='nameB']]");	

		assertNotNull( result );
		assertEquals( result.getName(),"child" );
		assertEquals( result.getElementByName("subchild").getAttributeValueByName("name"),"nameB" );
		
	}
	
	public void testSearchMatches()
	{
		XmlPathSearch search = new XmlPathSearch(testDocument);
		
		XmlElement result = search.findElement("root/child/subchild[@name?'.*B']");
		
		assertNotNull( result );
		assertEquals( result.getName(),"subchild" );
		assertEquals( result.getAttributeValueByName("name"),"nameB" );
		
	}
}
